@@ -9,20 +9,20 @@
 """a BoostBook formatter"""
 
 from Synopsis.Processor import Processor, Parameter
-from Synopsis import AST, Type, Util
+from Synopsis import ASG
 
-import sys, getopt, os, os.path, string, re
+import sys, getopt, os, os.path
 
-class Formatter(Processor, Type.Visitor, AST.Visitor):
+class Formatter(Processor, ASG.Visitor):
    """
    The type visitors should generate names relative to the current scope.
    The generated references however are fully scoped names
    """
 
-   def process(self, ast, **kwds):
+   def process(self, ir, **kwds):
       
       self.set_parameters(kwds)
-      self.ast = self.merge_input(ast)
+      self.ir = self.merge_input(ir)
 
       self.__os = open(self.output, 'w')
       self.__scope = ()
@@ -30,12 +30,12 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
       self.__indent = 0
       self.__type_label = ''
       
-      for d in ast.declarations():
+      for d in ir.asg.declarations:
          d.accept(self)
 
       self.__os.close()
       
-      return self.ast
+      return self.ir
 
    def scope(self): return self.__scope
    def push_scope(self, newscope):
@@ -99,51 +99,51 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
 
    def label(self, ref):
 
-      location = self.__toc.lookup(Util.ccolonName(ref))
-      ref = Util.ccolonName(ref, self.scope())
+      location = self.__toc.lookup(str(ref))
+      ref = str(self.scope().prune(ref))
       if location != "": return name("\"" + location + "\"", ref)
       else: return ref
 
    def type_label(self): return self.__type_label
    #################### Type Visitor ##########################################
 
-   def visitBaseType(self, type):
+   def visit_builtin_type_id(self, type):
 
-      self.__type_ref = Util.ccolonName(type.name())
-      self.__type_label = Util.ccolonName(type.name())
+      self.__type_ref = str(type.name)
+      self.__type_label = str(type.name)
         
-   def visitUnknown(self, type):
+   def visit_unknown_type_id(self, type):
 
-      self.__type_ref = Util.ccolonName(type.name())
-      self.__type_label = Util.ccolonName(type.name(), self.scope())
+      self.__type_ref = str(type.name)
+      self.__type_label = str(self.scope().prune(type.name))
 
-   def visitDeclared(self, type):
+   def visit_declared_type_id(self, type):
 
-      self.__type_label = Util.ccolonName(type.name(), self.scope())
-      self.__type_ref = Util.ccolonName(type.name())
+      self.__type_label = str(self.scope().prune(type.name))
+      self.__type_ref = str(type.name)
 
-   def visitModifier(self, type):
+   def visit_modifier_type_id(self, type):
 
-      type.alias().accept(self)
-      self.__type_ref = string.join(type.premod()) + " " + self.__type_ref + " " + string.join(type.postmod())
-      self.__type_label = string.join(type.premod()) + " " + self.__type_label + " " + string.join(type.postmod())
+      type.alias.accept(self)
+      self.__type_ref = ''.join(type.premod) + ' ' + self.__type_ref + ' ' + ''.join(type.postmod)
+      self.__type_label = ''.join(type.premod) + ' ' + self.__type_label + ' ' + ''.join(type.postmod)
 
-   def visitParametrized(self, type):
+   def visit_parametrized_type_id(self, type):
 
-      type.template().accept(self)
+      type.template.accept(self)
       type_label = self.__type_label + "&lt;"
       parameters_label = []
-      for p in type.parameters():
+      for p in type.parameters:
          p.accept(self)
          parameters_label.append(self.__type_label)
-      self.__type_label = type_label + string.join(parameters_label, ", ") + "&gt;"
+      self.__type_label = type_label + ', '.join(parameters_label) + '&gt;'
 
    def format_type(self, type):
 
       type.accept(self)
       return self.__type_label
 
-   def visitFunctionType(self, type):
+   def visit_function_type_id(self, type):
 
       # TODO: this needs to be implemented
       self.__type_ref = 'function_type'
@@ -154,58 +154,58 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
       text = doc.replace('\n\n', '</para><para>')
       self.write(self.entity("para", text)+'\n')
 
-   #################### AST Visitor ###########################################
+   #################### ASG Visitor ###########################################
          
-   def visitDeclarator(self, node):
-      self.__declarator = node.name()
-      for i in node.sizes():
-         self.__declarator[-1] = self.__declarator[-1] + "[" + str(i) + "]"
+   def visit_declarator(self, node):
+      self.__declarator = node.name
+      for i in node.sizes:
+         self.__declarator[-1] = self.__declarator[-1] + '[%d]'%i
 
-   def visitTypedef(self, typedef):
+   def visit_typedef(self, typedef):
 
-      self.start_entity("typedef", name=Util.ccolonName(self.scope(), typedef.name()))
-      self.write_entity("type", self.format_type(typedef.alias()))
+      self.start_entity("typedef", name=str(self.scope().prune(typedef.name)))
+      self.write_entity("type", self.format_type(typedef.alias))
       self.end_entity("typedef")
 
-   def visitVariable(self, variable):
+   def visit_variable(self, variable):
 
       self.start_entity("fieldsynopsis")
-      variable.vtype().accept(self)
+      variable.vtype.accept(self)
       self.entity("type", self.type_label())
-      self.entity("varname", variable.name()[-1])
+      self.entity("varname", variable.name[-1])
       self.end_entity("fieldsynopsis")
 
-   def visitConst(self, const):
+   def visit_const(self, const):
 
       print "sorry, <const> not implemented"
 
-   def visitModule(self, module):
+   def visit_module(self, module):
 
-      self.start_entity("namespace", name=Util.ccolonName(self.scope(), module.name()))
+      self.start_entity("namespace", name=str(self.scope().prune(module.name)))
       self.write("\n")
       self.process_doc(module.annotations.get('doc', ''))
-      self.push_scope(module.name())
-      for declaration in module.declarations(): declaration.accept(self)
+      self.push_scope(module.name)
+      for declaration in module.declarations: declaration.accept(self)
       self.pop_scope()
       self.end_entity("namespace")
 
-   def visitClass(self, clas):
+   def visit_class(self, class_):
 
-      self.start_entity("class", name=Util.ccolonName(self.scope(), clas.name()))
-      # clas.type()
-      if len(clas.parents()):
-         for parent in clas.parents(): parent.accept(self)
-      self.push_scope(clas.name())
-      if clas.annotations.has_key('doc'):
+      self.start_entity("class", name=str(self.scope().prune(class_.name)))
+      # class_.type
+      for p in class_.parents:
+         p.accept(self)
+      self.push_scope(class_.name)
+      if class_.annotations.has_key('doc'):
          self.start_entity("purpose")
-         self.process_doc(clas.annotations.get['doc'])
+         self.process_doc(class_.annotations.get['doc'])
          self.end_entity("purpose")
-      for declaration in clas.declarations():
-         declaration.accept(self)
+      for d in class_.declarations:
+         d.accept(self)
       self.pop_scope()
       self.end_entity("class")
 
-   def visitInheritance(self, inheritance):
+   def visit_inheritance(self, inheritance):
 
       if len(inheritance.attributes()):
          self.start_entity("inherit", access=inheritance.attributes()[0])
@@ -214,25 +214,25 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
       self.write_entity("classname", self.format_type(inheritance.parent()))
       self.end_entity("inherit")
 
-   def visitParameter(self, parameter):
+   def visit_parameter(self, parameter):
 
-      self.start_entity("parameter", name=parameter.identifier())
-      self.write_entity("param_type", self.format_type(parameter.type()))
-      #map(lambda m, this=self: this.write_entity("modifier", m), parameter.premodifier())
-      #map(lambda m, this=self: this.write_entity("modifier", m), parameter.postmodifier())
+      self.start_entity("parameter", name=parameter.name)
+      self.write_entity("param_type", self.format_type(parameter.type))
+      #map(lambda m, this=self: this.write_entity("modifier", m), parameter.premodifier)
+      #map(lambda m, this=self: this.write_entity("modifier", m), parameter.postmodifier)
       self.end_entity("parameter")
       self.write("\n")
 
-   def visitFunction(self, function):
+   def visit_function(self, function):
 
-      self.start_entity("function", name=Util.ccolonName(self.scope(), function.realname()))
+      self.start_entity("function", name=str(self.scope().prune(function.real_name)))
       self.do_function(function)
       self.end_entity("function")
       self.write("\n")
 
-   def visitOperation(self, operation):
+   def visit_operation(self, operation):
 
-      name = operation.name()
+      name = operation.name
       tag = None
       if len(name) > 1:
          if name[-1] == name[-2]:
@@ -243,7 +243,7 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
             self.start_entity(tag)
       if tag is None:
          tag = "method"
-         self.start_entity(tag, name=Util.ccolonName(self.scope(), operation.realname()))
+         self.start_entity(tag, name=str(self.scope().prune(operation.real_name)))
       self.do_function(operation)
       self.end_entity(tag)
       self.write("\n")
@@ -251,9 +251,9 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
    def do_function(self, func):
       """Stuff common to functions and methods, contructors, destructors"""
 
-      for parameter in func.parameters(): parameter.accept(self)
-      if func.returnType():
-         self.write_entity("type", self.format_type(func.returnType()))
+      for parameter in func.parameters: parameter.accept(self)
+      if func.return_type:
+         self.write_entity("type", self.format_type(func.return_type))
          self.write("\n")
       if func.annotations.has_key('doc'):
          self.start_entity("purpose")
@@ -261,18 +261,18 @@ class Formatter(Processor, Type.Visitor, AST.Visitor):
          self.end_entity("purpose")
          self.write("\n")
 	
-      if func.exceptions():
+      if func.exceptions:
          self.start_entity("throws")
-         for ex in func.exceptions():
+         for ex in func.exceptions:
             self.write_entity("simpara", ex)
          self.end_entity("throws")
          self.write("\n")
 
-   def visitEnumerator(self, enumerator):
+   def visit_enumerator(self, enumerator):
 
       print "sorry, <enumerator> not implemented"
 
-   def visitEnum(self, enum):
+   def visit_enum(self, enum):
 
       print "sorry, <enum> not implemented"
 
