@@ -11,10 +11,11 @@ Uses 'dot' from graphviz to generate various graphs.
 """
 
 from Synopsis.Processor import *
-from Synopsis import AST, Type, Util
+from Synopsis.QualifiedName import *
+from Synopsis import ASG
 from Synopsis.Formatters import TOC
-
-import sys, tempfile, getopt, os, os.path, string, types, errno, re
+from Synopsis.Formatters import quote_name, open_file
+import sys, os
 
 verbose = False
 debug = False
@@ -31,7 +32,7 @@ class SystemError:
 
    def __repr__(self):
 
-      return "SystemError: %(retval)x\"%(command)s\" failed."%self.__dict__
+      return 'SystemError: %(retval)x"%(command)s" failed.'%self.__dict__
 
 def system(command):
    """Run the command. If the command fails, an exception SystemError is
@@ -41,19 +42,31 @@ def system(command):
    if (ret>>8) != 0:
       raise SystemError(ret, command)
 
+
+def normalize(color):
+   """Generate a color triplet from a color string."""
+
+   if type(color) is str and color[0] == '#':
+      return (int(color[1:3], 16), int(color[3:5], 16), int(color[5:7], 16))
+   elif type(color) is tuple:
+      return (color[0] * 255, color[1] * 255, color[2] * 255)
+
+
+def light(color):
+
+   import colorsys
+   hsv = colorsys.rgb_to_hsv(*color)
+   return colorsys.hsv_to_rgb(hsv[0], hsv[1], hsv[2]/2)
+
+
 class DotFileGenerator:
    """A class that encapsulates the dot file generation"""
    def __init__(self, os, direction, bgcolor):
 
       self.__os = os
       self.direction = direction
-      self.bgcolor = bgcolor
-      # bgcolor is either a named color, or a hex value starting with '#', or a hsv triple
-      if self.bgcolor:
-         if type(self.bgcolor) is str:
-            if self.bgcolor[0] == '#': self.bgcolor = '"%s"'%self.bgcolor
-         if type(self.bgcolor) is tuple:
-            self.bgcolor = '"%f,%f,%f"'%(self.bgcolor[0], self.bgcolor[1], self.bgcolor[2])
+      self.bgcolor = bgcolor and '"#%X%X%X"'%bgcolor
+      self.light_color = bgcolor and '"#%X%X%X"'%light(bgcolor) or 'gray75'
       self.nodes = {}
 
    def write(self, text): self.__os.write(text)
@@ -66,28 +79,26 @@ class DotFileGenerator:
       number = self.nodes[name]
 
       # Quote to remove characters that dot can't handle
-      label = re.sub('<',r'\<',label)
-      label = re.sub('>',r'\>',label)
-      label = re.sub('{',r'\{',label)
-      label = re.sub('}',r'\}',label)
+      for p in [('<', '\<'), ('>', '\>'), ('{','\{'), ('}','\}')]:
+         label = label.replace(*p)
 
       if self.bgcolor:
          attr['fillcolor'] = self.bgcolor
          attr['style'] = 'filled'
 
       self.write("Node" + str(number) + " [shape=\"record\", label=\"{" + label + "}\"")
-      self.write(", fontSize = 10, height = 0.2, width = 0.4")
-      self.write(string.join(map(lambda item:', %s=%s'%item, attr.items())))
-      if ref: self.write(", URL=\"" + ref + "\"")
-      self.write("];\n")
+      #self.write(", fontSize = 10, height = 0.2, width = 0.4")
+      self.write(''.join([', %s=%s'%item for item in attr.items()]))
+      if ref: self.write(', URL="' + ref + '"')
+      self.write('];\n')
 
    def write_edge(self, parent, child, **attr):
 
       self.write("Node" + str(self.nodes[parent]) + " -> ")
       self.write("Node" + str(self.nodes[child]))
-      self.write("[ color=\"black\", fontsize=10, dir=back" + string.join(map(lambda item:', %s="%s"'%item, attr.items())) + "];\n")
+      self.write('[ color="black", fontsize=10, dir=back' + ''.join([', %s="%s"'%item for item in attr.items()]) + '];\n')
 
-class InheritanceGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
+class InheritanceGenerator(DotFileGenerator, ASG.Visitor):
    """A Formatter that generates an inheritance graph. If the 'toc' argument is not None,
    it is used to generate URLs. If no reference could be found in the toc, the node will
    be grayed out."""
@@ -101,14 +112,19 @@ class InheritanceGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
       else: self.__attributes = None
       self.aggregation = aggregation
       self.toc = toc
-      self.__scope = []
-      if prefix: self.__scope = string.split(prefix, '::')
+      self.scope = QualifiedName()
+      if prefix:
+         if prefix.contains('::'):
+            self.scope = QualifiedCxxName(prefix.split('::'))
+         elif prefix.contains('.'):
+            self.scope = QualifiedPythonName(prefix.split('.'))
+         else:
+            self.scope = QualifiedName((prefix,))
       self.__type_ref = None
       self.__type_label = ''
       self.__no_descend = no_descend
       self.nodes = {}
 
-   def scope(self): return self.__scope
    def type_ref(self): return self.__type_ref
    def type_label(self): return self.__type_label
    def parameter(self): return self.__parameter
@@ -129,109 +145,109 @@ class InheritanceGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
       """Returns the name of the given class node, relative to all its
       parents. This makes the graph simpler by making the names shorter"""
 
-      base = node.name()
-      for i in node.parents():
+      base = node.name
+      for i in node.parents:
          try:
-            parent = i.parent()
-            pname = parent.name()
+            parent = i.parent
+            pname = parent.name
             for j in range(len(base)):
                if j > len(pname) or pname[j] != base[j]:
                   # Base is longer than parent name, or found a difference
                   base[j:] = []
                   break
          except: pass # typedefs etc may cause errors here.. ignore
-      if not node.parents():
-         base = self.scope()
-      return Util.ccolonName(node.name(), base)
+      if not node.parents:
+         base = self.scope
+      return str(self.scope.prune(node.name))
 
    #################### Type Visitor ##########################################
 
-   def visitModifier(self, type):
+   def visit_modifier_type(self, type):
 
-      self.format_type(type.alias())
-      self.__type_label = string.join(type.premod()) + self.__type_label
-      self.__type_label = self.__type_label + string.join(type.postmod())
+      self.format_type(type.alias)
+      self.__type_label = ''.join(type.premod) + self.__type_label
+      self.__type_label = self.__type_label + ''.join(type.postmod)
 
-   def visitUnknown(self, type):
+   def visit_unknown_type(self, type):
 
-      self.__type_ref = self.toc and self.toc[type.link()] or None
-      self.__type_label = Util.ccolonName(type.name(), self.scope())
+      self.__type_ref = self.toc and self.toc[type.link] or None
+      self.__type_label = str(self.scope.prune(type.name))
         
-   def visitBase(self, type):
+   def visit_builtin_type_id(self, type):
 
       self.__type_ref = None
-      self.__type_label = type.name()[-1]
+      self.__type_label = type.name[-1]
 
-   def visitDependent(self, type):
+   def visit_dependent_type_id(self, type):
 
       self.__type_ref = None
-      self.__type_label = type.name()[-1]
+      self.__type_label = type.name[-1]
         
-   def visitDeclared(self, type):
+   def visit_declared_type_id(self, type):
 
-      self.__type_ref = self.toc and self.toc[type.declaration().name()] or None
-      if isinstance(type.declaration(), AST.Class):
-         self.__type_label = self.get_class_name(type.declaration())
+      self.__type_ref = self.toc and self.toc[type.declaration.name] or None
+      if isinstance(type.declaration, ASG.Class):
+         self.__type_label = self.get_class_name(type.declaration)
       else:
-         self.__type_label = Util.ccolonName(type.declaration().name(), self.scope())
+         self.__type_label = str(self.scope.prune(type.declaration.name))
 
-   def visitParametrized(self, type):
+   def visit_parametrized_type_id(self, type):
 
-      if type.template():
-         type_ref = self.toc and self.toc[type.template().name()] or None
-         type_label = Util.ccolonName(type.template().name(), self.scope())
+      if type.template:
+         type_ref = self.toc and self.toc[type.template.name] or None
+         type_label = str(self.scope.prune(type.template.name))
       else:
          type_ref = None
          type_label = "(unknown)"
       parameters_label = []
-      for p in type.parameters():
+      for p in type.parameters:
          parameters_label.append(self.format_type(p))
       self.__type_ref = type_ref
-      self.__type_label = type_label + "<" + string.join(parameters_label, ",") + ">"
+      self.__type_label = type_label + "<" + ','.join(parameters_label) + ">"
 
-   def visitTemplate(self, type):
+   def visit_template_id(self, type):
       self.__type_ref = None
       def clip(x, max=20):
          if len(x) > max: return '...'
          return x
-      self.__type_label = "template<%s>"%(clip(string.join(map(clip, map(self.format_type, type.parameters())), ","),40))
+      self.__type_label = "template<%s>"%(clip(','.join([clip(self.format_type(p)) for p in type.parameters]), 40))
 
-   #################### AST Visitor ###########################################
+   #################### ASG Visitor ###########################################
 
-   def visitInheritance(self, node):
+   def visit_inheritance(self, node):
 
-      self.format_type(node.parent())
+      self.format_type(node.parent)
       if self.type_ref():
          self.write_node(self.type_ref().link, self.type_label(), self.type_label())
       elif self.toc:
-         self.write_node('', self.type_label(), self.type_label(), color='gray75', fontcolor='gray75')
+         self.write_node('', self.type_label(), self.type_label(), color=self.light_color, fontcolor=self.light_color)
       else:
          self.write_node('', self.type_label(), self.type_label())
         
-   def visitClass(self, node):
+   def visit_class(self, node):
 
       if self.__operations is not None: self.__operations.append([])
       if self.__attributes is not None: self.__attributes.append([])
       name = self.get_class_name(node)
-      ref = self.toc and self.toc[node.name()] or None
-      for d in node.declarations(): d.accept(self)
+      ref = self.toc and self.toc[node.name] or None
+      for d in node.declarations: d.accept(self)
       # NB: old version of dot needed the label surrounded in {}'s (?)
       label = name
-      if node.template():
+      if type(node) is ASG.ClassTemplate and node.template:
          if self.direction == 'vertical':
-            label = self.format_type(node.template()) + '\\n' + label
+            label = self.format_type(node.template) + '\\n' + label
          else:
-            label = self.format_type(node.template()) + ' ' + label
+            label = self.format_type(node.template) + ' ' + label
       if self.__operations or self.__attributes:
          label = label + '\\n'
          if self.__operations:
-            label += '|' + string.join(map(lambda x:x[-1] + '()\\l', self.__operations[-1]), '')
+            label += '|' + ''.join([x[-1] + '()\\l' for x in self.__operations[-1]])
          if self.__attributes:
-            label += '|' + string.join(map(lambda x:x[-1] + '\\l', self.__attributes[-1]), '')
+            label += '|' + ''.join([x[-1] + '\\l' for x in self.__attributes[-1]])
       if ref:
          self.write_node(ref.link, name, label)
       elif self.toc:
-         self.write_node('', name, label, color='gray75', fontcolor='gray75')
+         self.write_node('', name, label, color=self.light_color, fontcolor=self.light_color)
       else:
          self.write_node('', name, label)
 
@@ -240,33 +256,33 @@ class InheritanceGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
          #       but also derived types such as pointers, references, STL containers, etc.
          #
          # find attributes of type 'Class' so we can link to it
-         for a in filter(lambda a:isinstance(a, AST.Variable), node.declarations()):
-            if isinstance(a.vtype(), Type.Declared):
-               d = a.vtype().declaration()
-               if isinstance(d, AST.Class) and self.nodes.has_key(self.get_class_name(d)):
+         for a in filter(lambda a:isinstance(a, ASG.Variable), node.declarations):
+            if isinstance(a.vtype, ASG.DeclaredTypeId):
+               d = a.vtype.declaration
+               if isinstance(d, ASG.Class) and self.nodes.has_key(self.get_class_name(d)):
                   self.write_edge(self.get_class_name(node), self.get_class_name(d),
                                   arrowtail='ediamond')
 
-      for inheritance in node.parents():
-         inheritance.accept(self)
+      for p in node.parents:
+         p.accept(self)
          self.write_edge(self.type_label(), name, arrowtail='empty')
       if self.__no_descend: return
       if self.__operations: self.__operations.pop()
       if self.__attributes: self.__attributes.pop()
 
-   def visitOperation(self, operation):
+   def visit_operation(self, operation):
 
       if self.__operations:
-         self.__operations[-1].append(operation.realname())
+         self.__operations[-1].append(operation.real_name)
 
-   def visitVariable(self, variable):
+   def visit_variable(self, variable):
 
       if self.__attributes:
-         self.__attributes[-1].append(variable.name())
+         self.__attributes[-1].append(variable.name)
 
 class SingleInheritanceGenerator(InheritanceGenerator):
    """A Formatter that generates an inheritance graph for a specific class.
-   This Visitor visits the AST upwards, i.e. following the inheritance links, instead of
+   This Visitor visits the ASG upwards, i.e. following the inheritance links, instead of
    the declarations contained in a given scope."""
 
    def __init__(self, os, direction, operations, attributes, levels, types,
@@ -280,28 +296,28 @@ class SingleInheritanceGenerator(InheritanceGenerator):
 
    #################### Type Visitor ##########################################
 
-   def visitDeclared(self, type):
+   def visit_declared_type_id(self, type):
       if self.__current < self.__levels or self.__levels == -1:
          self.__current = self.__current + 1
-         type.declaration().accept(self)
+         type.declaration.accept(self)
          self.__current = self.__current - 1
       # to restore the ref/label...
-      InheritanceGenerator.visitDeclared(self, type)
+      InheritanceGenerator.visit_declared_type_id(self, type)
 
-   #################### AST Visitor ###########################################
+   #################### ASG Visitor ###########################################
         
-   def visitInheritance(self, node):
+   def visit_inheritance(self, node):
 
-      node.parent().accept(self)
+      node.parent.accept(self)
       if self.type_label():
          if self.type_ref():
             self.write_node(self.type_ref().link, self.type_label(), self.type_label())
          elif self.toc:
-            self.write_node('', self.type_label(), self.type_label(), color='gray75', fontcolor='gray75')
+            self.write_node('', self.type_label(), self.type_label(), color=self.light_color, fontcolor=self.light_color)
          else:
             self.write_node('', self.type_label(), self.type_label())
         
-   def visitClass(self, node):
+   def visit_class(self, node):
 
       # Prevent recursion
       if self.__visited_classes.has_key(id(node)): return
@@ -309,18 +325,18 @@ class SingleInheritanceGenerator(InheritanceGenerator):
 
       name = self.get_class_name(node)
       if self.__current == 1:
-         self.write_node('', name, name, style='filled', color='lightgrey')
+         self.write_node('', name, name, style='filled', color=self.light_color, fontcolor=self.light_color)
       else:
-         ref = self.toc and self.toc[node.name()] or None
+         ref = self.toc and self.toc[node.name] or None
          if ref:
             self.write_node(ref.link, name, name)
          elif self.toc:
-            self.write_node('', name, name, color='gray75', fontcolor='gray75')
+            self.write_node('', name, name, color=self.light_color, fontcolor=self.light_color)
          else:
             self.write_node('', name, name)
 
-      for inheritance in node.parents():
-         inheritance.accept(self)
+      for p in node.parents:
+         p.accept(self)
          if self.nodes.has_key(self.type_label()):
             self.write_edge(self.type_label(), name, arrowtail='empty')
       # if this is the main class and if there is a type dictionary,
@@ -328,29 +344,29 @@ class SingleInheritanceGenerator(InheritanceGenerator):
 
       # if this is the main class
       if self.__current == 1 and self.__types:
-         # fool the visitDeclared method to stop walking upwards
+         # fool the visit_declared_type_id method to stop walking upwards
          self.__levels = 0
          for t in self.__types.values():
-            if isinstance(t, Type.Declared):
-               child = t.declaration()
-               if isinstance(child, AST.Class):
-                  for inheritance in child.parents():
-                     type = inheritance.parent()
+            if isinstance(t, ASG.DeclaredTypeId):
+               child = t.declaration
+               if isinstance(child, ASG.Class):
+                  for i in child.parents:
+                     type = i.parent
                      type.accept(self)
                      if self.type_ref():
-                        if self.type_ref().name == node.name():
+                        if self.type_ref().name == node.name:
                            child_label = self.get_class_name(child)
-                           ref = self.toc and self.toc[child.name()] or None
+                           ref = self.toc and self.toc[child.name] or None
                            if ref:
                               self.write_node(ref.link, child_label, child_label)
                            elif self.toc:
-                              self.write_node('', child_label, child_label, color='gray75', fontcolor='gray75')
+                              self.write_node('', child_label, child_label, color=self.light_color, fontcolor=self.light_color)
                            else:
                               self.write_node('', child_label, child_label)
 
                            self.write_edge(name, child_label, arrowtail='empty')
 
-class FileDependencyGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
+class FileDependencyGenerator(DotFileGenerator, ASG.Visitor):
    """A Formatter that generates a file dependency graph"""
 
    def visit_file(self, file):
@@ -367,12 +383,12 @@ class FileDependencyGenerator(DotFileGenerator, AST.Visitor, Type.Visitor):
 def _rel(frm, to):
    "Find link to to relative to frm"
 
-   frm = string.split(frm, '/'); to = string.split(to, '/')
+   frm = frm.split('/'); to = to.split('/')
    for l in range((len(frm)<len(to)) and len(frm)-1 or len(to)-1):
       if to[0] == frm[0]: del to[0]; del frm[0]
       else: break
    if frm: to = ['..'] * (len(frm) - 1) + to
-   return string.join(to,'/')
+   return '/'.join(to)
 
 def _convert_map(input, output, base_url):
    """convert map generated from Dot to a html region map.
@@ -382,9 +398,9 @@ def _convert_map(input, output, base_url):
    while line:
       line = line[:-1]
       if line[0:4] == "rect":
-         url, x1y1, x2y2 = string.split(line[4:])
-         x1, y1 = string.split(x1y1, ",")
-         x2, y2 = string.split(x2y2, ",")
+         url, x1y1, x2y2 = line[4:].split()
+         x1, y1 = x1y1.split(',')
+         x2, y2 = x2y2.split(',')
          output.write('<area alt="'+url+'" href="' + _rel(base_url, url) + '" shape="rect" coords="')
          output.write(str(x1) + ", " + str(y1) + ", " + str(x2) + ", " + str(y2) + '" />\n')
       line = input.readline()
@@ -413,7 +429,7 @@ def _format_html(input, output, base_url):
    _format(input, output + ".map", "imap")
    prefix, name = os.path.split(output)
    reference = name + ".png"
-   html = Util.open(output + ".html")
+   html = open_file(output + ".html")
    html.write('<img alt="'+name+'" src="' + reference + '" hspace="8" vspace="8" border="0" usemap="#')
    html.write(name + "_map\" />\n")
    html.write("<map name=\"" + name + "_map\">")
@@ -428,22 +444,29 @@ class Formatter(Processor):
    the various InheritanceGenerators"""
 
    title = Parameter('Inheritance Graph', 'the title of the graph')
-   type = Parameter('class', 'type of graph (one of \'file\', \'class\', \'single\'')
+   type = Parameter('class', 'type of graph (one of "file", "class", "single"')
    hide_operations = Parameter(True, 'hide operations')
    hide_attributes = Parameter(True, 'hide attributes')
    show_aggregation = Parameter(False, 'show aggregation')
    bgcolor = Parameter(None, 'background color for nodes')
-   format = Parameter('ps', "Generate output in format 'dot', 'ps', 'png', 'gif', 'map', 'html'")
+   format = Parameter('ps', 'Generate output in format "dot", "ps", "png", "svg", "gif", "map", "html"')
    layout = Parameter('vertical', 'Direction of graph')
    prefix = Parameter(None, 'Prefix to strip from all class names')
    toc_in = Parameter([], 'list of table of content files to use for symbol lookup')
    base_url = Parameter(None, 'base url to use for generated links')
 
-   def process(self, ast, **kwds):
+   def process(self, ir, **kwds):
       global verbose, debug
       
       self.set_parameters(kwds)
-      self.ast = self.merge_input(ast)
+      if self.bgcolor:
+         bgcolor = normalize(self.bgcolor)
+         if not bgcolor:
+            raise InvalidArgument('bgcolor=%s'%repr(self.bgcolor))
+         else:
+            self.bgcolor = bgcolor
+
+      self.ir = self.merge_input(ir)
       verbose = self.verbose
       debug = self.debug
 
@@ -451,14 +474,15 @@ class Formatter(Processor):
                  'ps' : 'ps',
                  'png' : 'png',
                  'gif' : 'gif',
+                 'svg' : 'svg',
                  'map' : 'imap',
                  'html' : 'html'}
 
       if formats.has_key(self.format): format = formats[self.format]
       else:
          print "Error: Unknown format. Available formats are:",
-         print string.join(formats.keys(), ', ')
-         return self.ast
+         print ', '.join(formats.keys())
+         return self.ir
 
       # we only need the toc if format=='html'
       if format == 'html':
@@ -469,9 +493,9 @@ class Formatter(Processor):
          toc = None
 
       head, tail = os.path.split(self.output)
-      tmpfile = os.path.join(head, Util.quote(tail)) + ".dot"
+      tmpfile = os.path.join(head, quote_name(tail)) + ".dot"
       if self.verbose: print "Dot Formatter: Writing dot file..."
-      dotfile = Util.open(tmpfile)
+      dotfile = open_file(tmpfile)
       dotfile.write("digraph \"%s\" {\n"%(self.title))
       if self.layout == 'horizontal':
          dotfile.write('rankdir="LR";\n')
@@ -481,7 +505,7 @@ class Formatter(Processor):
          generator = SingleInheritanceGenerator(dotfile, self.layout,
                                                 not self.hide_operations,
                                                 not self.hide_attributes,
-                                                -1, self.ast.types(),
+                                                -1, self.ir.asg.types,
                                                 toc, self.prefix, False,
                                                 self.bgcolor)
       elif self.type == 'class':
@@ -498,10 +522,10 @@ class Formatter(Processor):
          
 
       if self.type == 'file':
-         for f in self.ast.files().values():
+         for f in self.ir.files.values():
             generator.visit_file(f)
       else:
-         for d in self.ast.declarations():
+         for d in self.ir.asg.declarations:
             d.accept(generator)
       dotfile.write("}\n")
       dotfile.close()
@@ -517,5 +541,5 @@ class Formatter(Processor):
          _format(tmpfile, self.output, format)
          os.remove(tmpfile)
 
-      return self.ast
+      return self.ir
 
