@@ -8,20 +8,19 @@
 
 from Synopsis.Formatters.HTML.Tags import *
 from Default import Default
-from Synopsis import Util
 
 class InheritanceFormatter(Default):
    """Prints just the name of each declaration, with a link to its doc"""
 
    def format_declaration(self, decl, label=None):
 
-      if not label: label = decl.name()[-1]
-      fullname = Util.ccolonName(decl.name(), self.formatter.scope())
-      title = decl.type() + " " + escape(fullname)
-      return self.reference(decl.name(), label=label, title=title) + ' '
+      if not label: label = decl.name[-1]
+      qname = self.formatter.scope().prune(decl.name)
+      title = decl.type + ' ' + escape(str(qname))
+      return self.reference(decl.name, label=label, title=title) + ' '
 
    def format_function(self, decl):
 
-      return self.format_declaration(decl, label=decl.realname()[-1])
+      return self.format_declaration(decl, label=decl.real_name[-1])
 
    def format_operation(self, decl): return self.format_function(decl)
