@@ -7,8 +7,7 @@
 #
 
 from Synopsis.Processor import Parameter
-from Synopsis import AST, Util
-from Synopsis.FileTree import FileTree
+from Synopsis import FileTree
 from Synopsis.Formatters.HTML.View import View
 from Synopsis.Formatters.HTML.Tags import *
 
@@ -48,7 +47,7 @@ class FileListing(View):
       self.write_navigation_bar()
       self.write('<ul class="tree">')
       # recursively visit all nodes
-      self.process_file_tree_node(self.directory_tree.root())
+      self.process_file_tree_node(self.processor.file_tree.root())
       self.write('</ul>')
       self.end_file()
 
@@ -57,7 +56,7 @@ class FileListing(View):
       or not"""
 
       a_leaf = isinstance(a, FileTree.File)
-      b_leaf = isinstance(a, FileTree.File)
+      b_leaf = isinstance(b, FileTree.File)
       if a_leaf != b_leaf:
          return cmp(b_leaf, a_leaf)
       return cmp(a.path.upper(), b.path.upper())
