@@ -5,7 +5,7 @@
 // see the file COPYING for details.
 //
 
-#include <Synopsis/AST/ASTKit.hh>
+#include <Synopsis/ASG/ASGKit.hh>
 #include <Synopsis/Python/Module.hh>
 #include <Synopsis/Trace.hh>
 #include <Synopsis/PTree.hh>
@@ -15,7 +15,8 @@
 #include <Synopsis/Lexer.hh>
 #include <Synopsis/SymbolFactory.hh>
 #include <Synopsis/Parser.hh>
-#include "ASTTranslator.hh"
+#include "ASGTranslator.hh"
+// #include "SXRGenerator.hh"
 #include <Support/ErrorHandler.hh>
 #include <fstream>
 
@@ -34,25 +35,23 @@ void unexpected()
 
 PyObject *parse(PyObject * /* self */, PyObject *args)
 {
-  PyObject *py_ast;
+  PyObject *py_ir;
   const char *src, *cppfile;
   char const * base_path = "";
-  char const * syntax_prefix = 0;
-  char const * xref_prefix = 0;
+  char const * sxr_prefix = 0;
   int primary_file_only, verbose, debug;
-  if (!PyArg_ParseTuple(args, "Ossizzzii",
-                        &py_ast, &cppfile, &src,
+  if (!PyArg_ParseTuple(args, "Ossizzii",
+                        &py_ir, &cppfile, &src,
                         &primary_file_only,
                         &base_path,
-                        &syntax_prefix,
-                        &xref_prefix,
+                        &sxr_prefix,
                         &verbose,
                         &debug))
     return 0;
 
-  Py_INCREF(py_ast);
-  AST::AST ast(py_ast);
-  Py_INCREF(py_ast);
+  Py_INCREF(py_ir);
+  IR ir(py_ir);
+  Py_INCREF(py_ir);
 
   std::set_unexpected(unexpected);
   ErrorHandler error_handler();
@@ -74,24 +73,29 @@ PyObject *parse(PyObject * /* self */, PyObject *args)
     Parser parser(lexer, symbols, Parser::GCC);
     PTree::Node *ptree = parser.parse();
     const Parser::ErrorList &errors = parser.errors();
-    if (!errors.size() && ptree)
-    {
-      ASTTranslator translator(src, base_path, primary_file_only, ast, verbose, debug);
-      translator.translate(ptree, buffer);
-    }
-    else
+    if (!errors.empty())
     {
       for (Parser::ErrorList::const_iterator i = errors.begin(); i != errors.end(); ++i)
 	(*i)->write(std::cerr);
       throw std::runtime_error("The input contains errors.");
     }
+    else if (ptree)
+    {
+      ASGTranslator translator(src, base_path, primary_file_only, ir, verbose, debug);
+      translator.translate(ptree, buffer);
+//       if (sxr_prefix)
+//       {
+//         SXRGenerator sxr_generator(src, base_path, primary_file_only, ir, verbose, debug);
+//         srx_generator.generate(ptree, buffer);
+//       }
+    }
   }
   catch (std::exception const &e)
   {
     PyErr_SetString(error, e.what());
     return 0;
   }
-  return py_ast;
+  return py_ir;
 }
 
 PyMethodDef methods[] = {{(char*)"parse", parse, METH_VARARGS},
