@@ -5,7 +5,8 @@
 # see the file COPYING for details.
 #
 
-from Synopsis import AST
+from Synopsis import ASG
+from Synopsis.QualifiedName import QualifiedName
 from Synopsis.Processors.Transformer import Transformer
 import re
 
@@ -30,11 +31,11 @@ class Grouper(Transformer):
             print 'Warning: group stack is non-empty !'
             while (self.__group_stack[-1]):
                 group = self.__group_stack[-1][-1]
-                print 'forcing closing of group %s (opened near %s:%d)'%(group.name(), group.file().name, group.line())
+                print 'forcing closing of group %s (opened near %s:%d)'%(group.name, group.file.name, group.line)
                 self.pop_group()
 
     def finalize(self):
-        """replace the AST with the newly created one"""
+        """replace the ASG with the newly created one"""
 
         self.strip_dangling_groups()
         super(Grouper, self).finalize()
@@ -67,11 +68,11 @@ class Grouper(Transformer):
 
         if self.__group_stack[-1]:
             group = self.__group_stack[-1].pop()
-            group.declarations()[:] = self.current_scope()
+            group.declarations = self.current_scope()
             Transformer.pop(self, group)
         else:
             if decl:
-                print "Warning: no group open in current scope (near %s:%d), ignoring."%(decl.file().name, decl.line())
+                print "Warning: no group open in current scope (near %s:%d), ignoring."%(decl.file.name, decl.line)
             else:
                 print "Warning: no group open in current scope, ignoring."
 
@@ -94,15 +95,16 @@ class Grouper(Transformer):
             elif tag.group('open'):
 
                 if self.debug:
-                    print 'found group open tag in', decl.name()
+                    print 'found group open tag in', decl.name
 
                 # Open the group. <name> is remainder of line.
                 label = tag.group('name') or 'unnamed'
+                label = label.strip()
                 # The comment before the open marker becomes the group comment.
                 if tag.start('open') > 0:
                     c = c[:tag.start('open')]
                     comments.append(c)
-                group = AST.Group(decl.file(), decl.line(), 'group', [label])
+                group = ASG.Group(decl.file, decl.line, 'group', QualifiedName((label,)))
                 group.annotations['comments'] = comments
                 comments = []
                 self.push_group(group)
@@ -110,19 +112,19 @@ class Grouper(Transformer):
             elif tag.group('close'):
 
                 if self.debug:
-                    print 'found group close tag in', decl.name()
+                    print 'found group close tag in', decl.name
 
                 self.pop_group(decl)
 
         decl.annotations['comments'] = comments
 
 
-    def visitDeclaration(self, decl):
+    def visit_declaration(self, decl):
 
         self.process_comments(decl)
         self.add(decl)
         
-    def visitScope(self, scope):
+    def visit_scope(self, scope):
         """Visits all children of the scope in a new scope. The value of
         current_scope() at the end of the list is used to replace scope's list of
         declarations - hence you can remove (or insert) declarations from the
@@ -130,24 +132,25 @@ class Grouper(Transformer):
 
         self.process_comments(scope)
         self.push()
-        for decl in scope.declarations(): decl.accept(self)
-        scope.declarations()[:] = self.current_scope()
+        for d in scope.declarations: d.accept(self)
+        scope.declarations = self.current_scope()
         self.pop(scope)
 
-    def visitEnum(self, enum):
-        """Does the same as visitScope, but for the enum's list of
+    def visit_enum(self, enum):
+        """Does the same as visit_scope, but for the enum's list of
         enumerators"""
 
         self.process_comments(enum)
         self.push()
-        for enumor in enum.enumerators(): enumor.accept(self)
-        enum.enumerators()[:] = self.current_scope()
+        for enumor in enum.enumerators:
+            enumor.accept(self)
+        enum.enumerators = self.current_scope()
         self.pop(enum)
 
-    def visitEnumerator(self, enumor):
+    def visit_enumerator(self, enumor):
         """Removes dummy enumerators"""
 
-        if enumor.type() == "dummy": return #This wont work since Core.AST.Enumerator forces type to "enumerator"
-        if not len(enumor.name()): return # workaround.
+        if enumor.type == "dummy": return #This wont work since Core.ASG.Enumerator forces type to "enumerator"
+        if not len(enumor.name): return # workaround.
         self.add(enumor)
 
