@@ -6,28 +6,28 @@
 #
 
 from Synopsis.Processor import Processor, Parameter
-from Synopsis import AST
+from Synopsis import ASG
 
-class ModuleSorter(Processor, AST.Visitor):
+class ModuleSorter(Processor, ASG.Visitor):
     """A processor that sorts declarations in a module alphabetically."""
 
-    def process(self, ast, **kwds):
+    def process(self, ir, **kwds):
       
         self.set_parameters(kwds)
-        self.ast = self.merge_input(ast)
+        self.ir = self.merge_input(ir)
 
         self.__scopestack = []
         self.__currscope = []
 
-        for decl in self.ast.declarations():
+        for decl in self.ir.asg.declarations:
             decl.accept(self)
 
-        return self.output_and_return_ast()
+        return self.output_and_return_ir()
 
 
-    def visitMetaModule(self, module):
+    def visit_meta_module(self, module):
         """Visits all children of the module, and if there are no declarations
         after that removes the module"""
 
-        def compare(a, b): return cmp(a.name(), b.name())
-        module.declarations().sort(compare)
+        def compare(a, b): return cmp(a.name, b.name)
+        module.declarations.sort(compare)
