@@ -6,7 +6,7 @@
 #
 
 from Processor import Processor, Error
-import AST
+import IR
 from getoptions import get_options
 
 import sys
@@ -60,15 +60,15 @@ def process(argv = sys.argv, **commands):
 
     props = {}
     # process all option arguments...
-    for o, a in get_options(args): props[o] = a
+    for o, a in get_options(args): props[o.replace('-', '_')] = a
 
     # ...and keep remaining (non-option) arguments as 'input'
     if args: props['input'] = args
 
     if command in commands:
-        ast = AST.AST()
+        ir = IR.IR()
         try:
-            commands[command].process(ast, **props)
+            commands[command].process(ir, **props)
         except Error, e:
             error(str(e))
         except KeyboardInterrupt, e:
