@@ -26,8 +26,9 @@
  * None of this is safe with dlclose and incremental collection.
  * But then not much of anything is safe in the presence of dlclose.
  */
-#if defined(__linux__) && !defined(_GNU_SOURCE)
-    /* Can't test LINUX, since this must be define before other includes */
+#if (defined(__linux__) || defined(__GLIBC__) || defined(__GNU__)) \
+     && !defined(_GNU_SOURCE)
+    /* Can't test LINUX, since this must be defined before other includes */
 #   define _GNU_SOURCE
 #endif
 #if !defined(MACOS) && !defined(_WIN32_WCE)
@@ -49,35 +50,36 @@
 #   undef GC_must_restore_redefined_dlopen
 # endif
 
+/* A user-supplied routine that is called to determine if a DSO must
+   be scanned by the gc.  */
+static int (*GC_has_static_roots)(const char *, void *, size_t);
+
+
 #if (defined(DYNAMIC_LOADING) || defined(MSWIN32) || defined(MSWINCE)) \
     && !defined(PCR)
-#if !defined(SUNOS4) && !defined(SUNOS5DL) && !defined(IRIX5) && \
+#if !defined(SOLARISDL) && !defined(IRIX5) && \
     !defined(MSWIN32) && !defined(MSWINCE) && \
     !(defined(ALPHA) && defined(OSF1)) && \
     !defined(HPUX) && !(defined(LINUX) && defined(__ELF__)) && \
-    !defined(RS6000) && !defined(SCO_ELF) && !defined(DGUX) && \
+    !defined(AIX) && !defined(SCO_ELF) && !defined(DGUX) && \
     !(defined(FREEBSD) && defined(__ELF__)) && \
     !(defined(NETBSD) && defined(__ELF__)) && !defined(HURD) && \
-    !defined(MACOSX)
+    !defined(DARWIN) && !defined(CYGWIN32)
  --> We only know how to find data segments of dynamic libraries for the
  --> above.  Additional SVR4 variants might not be too
  --> hard to add.
 #endif
 
 #include <stdio.h>
-#ifdef SUNOS5DL
+#ifdef SOLARISDL
 #   include <sys/elf.h>
 #   include <dlfcn.h>
 #   include <link.h>
 #endif
-#ifdef SUNOS4
-#   include <dlfcn.h>
-#   include <link.h>
-#   include <a.out.h>
-  /* struct link_map field overrides */
-#   define l_next	lm_next
-#   define l_addr	lm_addr
-#   define l_name	lm_name
+
+#if defined(NETBSD)
+#   include <machine/elf_machdep.h>
+#   define ELFSIZE ARCH_ELFSIZE
 #endif
 
 #if defined(LINUX) && defined(__ELF__) || defined(SCO_ELF) || \
@@ -91,14 +93,28 @@
 /* Newer versions of GNU/Linux define this macro.  We
  * define it similarly for any ELF systems that don't.  */
 #  ifndef ElfW
-#    if !defined(ELF_CLASS) || ELF_CLASS == ELFCLASS32
-#      define ElfW(type) Elf32_##type
+#    if defined(FREEBSD)
+#      if __ELF_WORD_SIZE == 32
+#        define ElfW(type) Elf32_##type
+#      else
+#        define ElfW(type) Elf64_##type
+#      endif
+#    elif defined(NETBSD)
+#      if ELFSIZE == 32
+#        define ElfW(type) Elf32_##type
+#      else
+#        define ElfW(type) Elf64_##type
+#      endif
 #    else
-#      define ElfW(type) Elf64_##type
+#      if !defined(ELF_CLASS) || ELF_CLASS == ELFCLASS32
+#        define ElfW(type) Elf32_##type
+#      else
+#        define ElfW(type) Elf64_##type
+#      endif
 #    endif
 #  endif
 
-#if defined(SUNOS5DL) && !defined(USE_PROC_FOR_LIBRARIES)
+#if defined(SOLARISDL) && !defined(USE_PROC_FOR_LIBRARIES)
 
 #ifdef LINT
     Elf32_Dyn _DYNAMIC;
@@ -144,59 +160,17 @@ GC_FirstDLOpenedLinkMap()
     return cachedResult;
 }
 
-#endif /* SUNOS5DL ... */
+#endif /* SOLARISDL ... */
 
 /* BTL: added to fix circular dlopen definition if GC_SOLARIS_THREADS defined */
 # if defined(GC_must_restore_redefined_dlopen)
 #   define dlopen GC_dlopen
 # endif
 
-#if defined(SUNOS4) && !defined(USE_PROC_FOR_LIBRARIES)
-
-#ifdef LINT
-    struct link_dynamic _DYNAMIC;
-#endif
-
-static struct link_map *
-GC_FirstDLOpenedLinkMap()
-{
-    extern struct link_dynamic _DYNAMIC;
-
-    if( &_DYNAMIC == 0) {
-        return(0);
-    }
-    return(_DYNAMIC.ld_un.ld_1->ld_loaded);
-}
-
-/* Return the address of the ld.so allocated common symbol	*/
-/* with the least address, or 0 if none.			*/
-static ptr_t GC_first_common()
-{
-    ptr_t result = 0;
-    extern struct link_dynamic _DYNAMIC;
-    struct rtc_symb * curr_symbol;
-    
-    if( &_DYNAMIC == 0) {
-        return(0);
-    }
-    curr_symbol = _DYNAMIC.ldd -> ldd_cp;
-    for (; curr_symbol != 0; curr_symbol = curr_symbol -> rtc_next) {
-        if (result == 0
-            || (ptr_t)(curr_symbol -> rtc_sp -> n_value) < result) {
-            result = (ptr_t)(curr_symbol -> rtc_sp -> n_value);
-        }
-    }
-    return(result);
-}
-
-#endif  /* SUNOS4 ... */
-
-# if defined(SUNOS4) || defined(SUNOS5DL)
+# if defined(SOLARISDL)
 /* Add dynamic library data sections to the root set.		*/
 # if !defined(PCR) && !defined(GC_SOLARIS_THREADS) && defined(THREADS)
-#   ifndef SRC_M3
 	--> fix mutual exclusion with dlopen
-#   endif  /* We assume M3 programs don't call dlopen for now */
 # endif
 
 # ifndef USE_PROC_FOR_LIBRARIES
@@ -208,16 +182,6 @@ void GC_register_dynamic_libraries()
   for (lm = GC_FirstDLOpenedLinkMap();
        lm != (struct link_map *) 0;  lm = lm->l_next)
     {
-#     ifdef SUNOS4
-	struct exec *e;
-	 
-        e = (struct exec *) lm->lm_addr;
-        GC_add_roots_inner(
-      		    ((char *) (N_DATOFF(*e) + lm->lm_addr)),
-		    ((char *) (N_BSSADDR(*e) + e->a_bss + lm->lm_addr)),
-		    TRUE);
-#     endif
-#     ifdef SUNOS5DL
 	ElfW(Ehdr) * e;
         ElfW(Phdr) * p;
         unsigned long offset;
@@ -244,25 +208,11 @@ void GC_register_dynamic_libraries()
               break;
           }
 	}
-#     endif
     }
-#   ifdef SUNOS4
-      {
-      	static ptr_t common_start = 0;
-      	ptr_t common_end;
-      	extern ptr_t GC_find_limit();
-      	
-      	if (common_start == 0) common_start = GC_first_common();
-      	if (common_start != 0) {
-      	    common_end = GC_find_limit(common_start, TRUE);
-      	    GC_add_roots_inner((char *)common_start, (char *)common_end, TRUE);
-      	}
-      }
-#   endif
 }
 
 # endif /* !USE_PROC ... */
-# endif /* SUNOS */
+# endif /* SOLARISDL */
 
 #if defined(LINUX) && defined(__ELF__) || defined(SCO_ELF) || \
     (defined(FREEBSD) && defined(__ELF__)) || defined(DGUX) || \
@@ -283,92 +233,64 @@ extern ssize_t GC_repeat_read(int fd, char *buf, size_t count);
 	/* Repeatedly read until buffer is filled, or EOF is encountered */
 	/* Defined in os_dep.c.  					 */
 
-static char *parse_map_entry(char *buf_ptr, word *start, word *end,
-                             char *prot_buf, unsigned int *maj_dev);
+char *GC_parse_map_entry(char *buf_ptr, ptr_t *start, ptr_t *end,
+                         char **prot, unsigned int *maj_dev,
+			 char **mapping_name);
+char *GC_get_maps(void);
+	/* From os_dep.c	*/
 
-void GC_register_dynamic_libraries()
+word GC_register_map_entries(char *maps)
 {
-    int f;
-    int result;
-    char prot_buf[5];
-    int maps_size;
-    char maps_temp[32768];
-    char *maps_buf;
-    char *buf_ptr;
+    char *prot;
+    char *buf_ptr = maps;
     int count;
-    word start, end;
-    unsigned int maj_dev, min_dev;
-    word least_ha, greatest_ha;
+    ptr_t start, end;
+    unsigned int maj_dev;
+    ptr_t least_ha, greatest_ha;
     unsigned i;
-    word datastart = (word)(DATASTART);
+    ptr_t datastart = (ptr_t)(DATASTART);
 
-    /* Read /proc/self/maps	*/
-        /* Note that we may not allocate, and thus can't use stdio.	*/
-        f = open("/proc/self/maps", O_RDONLY);
-        if (-1 == f) ABORT("Couldn't open /proc/self/maps");
-	/* stat() doesn't work for /proc/self/maps, so we have to
-	   read it to find out how large it is... */
-	maps_size = 0;
-	do {
-	    result = GC_repeat_read(f, maps_temp, sizeof(maps_temp));
-	    if (result <= 0) ABORT("Couldn't read /proc/self/maps");
-	    maps_size += result;
-	} while (result == sizeof(maps_temp));
-
-	if (maps_size > sizeof(maps_temp)) {
-	    /* If larger than our buffer, close and re-read it. */
-	    close(f);
-	    f = open("/proc/self/maps", O_RDONLY);
-	    if (-1 == f) ABORT("Couldn't open /proc/self/maps");
-	    maps_buf = alloca(maps_size);
-	    if (NULL == maps_buf) ABORT("/proc/self/maps alloca failed");
-	    result = GC_repeat_read(f, maps_buf, maps_size);
-	    if (result <= 0) ABORT("Couldn't read /proc/self/maps");
-	} else {
-	    /* Otherwise use the fixed size buffer */
-	    maps_buf = maps_temp;
-	}
-
-	close(f);
-        maps_buf[result] = '\0';
-        buf_ptr = maps_buf;
-    /* Compute heap bounds. Should be done by add_to_heap?	*/
-	least_ha = (word)(-1);
+    /* Compute heap bounds. FIXME: Should work if heap and roots are 	*/
+    /* interleaved?							*/
+	least_ha = (ptr_t)(word)(-1);
 	greatest_ha = 0;
 	for (i = 0; i < GC_n_heap_sects; ++i) {
-	    word sect_start = (word)GC_heap_sects[i].hs_start;
-	    word sect_end = sect_start + GC_heap_sects[i].hs_bytes;
+	    ptr_t sect_start = GC_heap_sects[i].hs_start;
+	    ptr_t sect_end = sect_start + GC_heap_sects[i].hs_bytes;
 	    if (sect_start < least_ha) least_ha = sect_start;
 	    if (sect_end > greatest_ha) greatest_ha = sect_end;
         }
-    	if (greatest_ha < (word)GC_scratch_last_end_ptr)
-	    greatest_ha = (word)GC_scratch_last_end_ptr; 
+    	if (greatest_ha < (ptr_t)GC_scratch_last_end_ptr)
+	    greatest_ha = (ptr_t)GC_scratch_last_end_ptr; 
+
     for (;;) {
-
-        buf_ptr = parse_map_entry(buf_ptr, &start, &end, prot_buf, &maj_dev);
-	if (buf_ptr == NULL) return;
-
-	if (prot_buf[1] == 'w') {
+        buf_ptr = GC_parse_map_entry(buf_ptr, &start, &end, &prot, &maj_dev, 0);
+	if (buf_ptr == NULL) return 1;
+	if (prot[1] == 'w') {
 	    /* This is a writable mapping.  Add it to		*/
 	    /* the root set unless it is already otherwise	*/
 	    /* accounted for.					*/
-	    if (start <= (word)GC_stackbottom && end >= (word)GC_stackbottom) {
+	    if (start <= GC_stackbottom && end >= GC_stackbottom) {
 		/* Stack mapping; discard	*/
 		continue;
 	    }
 #	    ifdef THREADS
+	      /* This may fail, since a thread may already be 		*/
+	      /* unregistered, but its thread stack may still be there.	*/
+	      /* That can fail because the stack may disappear while	*/
+	      /* we're marking.  Thus the marker is, and has to be	*/
+	      /* prepared to recover from segmentation faults.		*/
 	      if (GC_segment_is_thread_stack(start, end)) continue;
+	      /* FIXME: REDIRECT_MALLOC actually works with threads on	*/
+	      /* LINUX/IA64 if we omit this check.  The problem is that	*/
+	      /* thread stacks contain pointers to dynamic thread	*/
+	      /* vectors, which may be reused due to thread caching.	*/
+	      /* Currently they may not be marked if the thread is 	*/
+	      /* still live.						*/
+	      /* For dead threads, we trace the whole stack, which is	*/
+	      /* very suboptimal for performance reasons.		*/
 #	    endif
-	    /* The rest of this assumes that there is no mapping	*/
-	    /* spanning the beginning of the data segment, or extending	*/
-	    /* beyond the entire heap at both ends.  			*/
-	    /* Empirically these assumptions hold.			*/
-	    
-	    if (start < (word)DATAEND && end > (word)DATAEND) {
-		/* Rld may use space at the end of the main data 	*/
-		/* segment.  Thus we add that in.			*/
-		start = (word)DATAEND;
-	    }
+	    /* We no longer exclude the main data segment.		*/
 	    if (start < least_ha && end > least_ha) {
 		end = least_ha;
 	    }
@@ -378,83 +300,23 @@ void GC_register_dynamic_libraries()
 	    if (start >= least_ha && end <= greatest_ha) continue;
 	    GC_add_roots_inner((char *)start, (char *)end, TRUE);
 	}
-     }
+    }
+    return 1;
+}
+
+void GC_register_dynamic_libraries()
+{
+    if (!GC_register_map_entries(GC_get_maps()))
+        ABORT("Failed to read /proc for library registration.");
 }
 
 /* We now take care of the main data segment ourselves: */
 GC_bool GC_register_main_static_data()
 {
-  return FALSE;
+    return FALSE;
 }
   
 # define HAVE_REGISTER_MAIN_STATIC_DATA
-//
-//  parse_map_entry parses an entry from /proc/self/maps so we can
-//  locate all writable data segments that belong to shared libraries.
-//  The format of one of these entries and the fields we care about
-//  is as follows:
-//  XXXXXXXX-XXXXXXXX r-xp 00000000 30:05 260537     name of mapping...\n
-//  ^^^^^^^^ ^^^^^^^^ ^^^^          ^^
-//  start    end      prot          maj_dev
-//  0        9        18            32
-//  
-//  For 64 bit ABIs:
-//  0	     17	      34	    56
-//
-//  The parser is called with a pointer to the entry and the return value
-//  is either NULL or is advanced to the next entry(the byte after the
-//  trailing '\n'.)
-//
-#if CPP_WORDSZ == 32
-# define OFFSET_MAP_START   0
-# define OFFSET_MAP_END     9
-# define OFFSET_MAP_PROT   18
-# define OFFSET_MAP_MAJDEV 32
-# define ADDR_WIDTH 	    8
-#endif
-
-#if CPP_WORDSZ == 64
-# define OFFSET_MAP_START   0
-# define OFFSET_MAP_END    17
-# define OFFSET_MAP_PROT   34
-# define OFFSET_MAP_MAJDEV 56
-# define ADDR_WIDTH 	   16
-#endif
-
-static char *parse_map_entry(char *buf_ptr, word *start, word *end,
-                             char *prot_buf, unsigned int *maj_dev)
-{
-    int i;
-    char *tok;
-
-    if (buf_ptr == NULL || *buf_ptr == '\0') {
-        return NULL;
-    }
-
-    memcpy(prot_buf, buf_ptr+OFFSET_MAP_PROT, 4); // do the protections first
-    prot_buf[4] = '\0';
-
-    if (prot_buf[1] == 'w') { // we can skip all of this if it's not writable
-
-        tok = buf_ptr;
-        buf_ptr[OFFSET_MAP_START+ADDR_WIDTH] = '\0';
-        *start = strtoul(tok, NULL, 16);
-
-        tok = buf_ptr+OFFSET_MAP_END;
-        buf_ptr[OFFSET_MAP_END+ADDR_WIDTH] = '\0';
-        *end = strtoul(tok, NULL, 16);
-
-        buf_ptr += OFFSET_MAP_MAJDEV;
-        tok = buf_ptr;
-        while (*buf_ptr != ':') buf_ptr++;
-        *buf_ptr++ = '\0';
-        *maj_dev = strtoul(tok, NULL, 16);
-    }
-
-    while (*buf_ptr && *buf_ptr++ != '\n');
-
-    return buf_ptr;
-}
 
 #endif /* USE_PROC_FOR_LIBRARIES */
 
@@ -463,7 +325,7 @@ static char *parse_map_entry(char *buf_ptr, word *start, word *end,
 /* For glibc 2.2.4+.  Unfortunately, it doesn't work for older	*/
 /* versions.  Thanks to Jakub Jelinek for most of the code.	*/
 
-# if defined(LINUX) /* Are others OK here, too? */ \
+# if (defined(LINUX) || defined (__GLIBC__)) /* Are others OK here, too? */ \
      && (__GLIBC__ > 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ > 2) \
          || (__GLIBC__ == 2 && __GLIBC_MINOR__ == 2 && defined(DT_CONFIG))) 
 
@@ -493,6 +355,11 @@ static int GC_register_dynlib_callback(info, size, ptr)
 	{
 	  if( !(p->p_flags & PF_W) ) break;
 	  start = ((char *)(p->p_vaddr)) + info->dlpi_addr;
+
+	  if (GC_has_static_roots
+	      && !GC_has_static_roots(info->dlpi_name, start, p->p_memsz))
+	    break;
+
 	  GC_add_roots_inner(start, start + p->p_memsz, TRUE);
 	}
       break;
@@ -549,6 +416,16 @@ GC_bool GC_register_main_static_data()
 
 #if defined(NETBSD)
 #  include <sys/exec_elf.h>
+/* for compatibility with 1.4.x */
+#  ifndef DT_DEBUG
+#  define DT_DEBUG     21
+#  endif
+#  ifndef PT_LOAD
+#  define PT_LOAD      1
+#  endif
+#  ifndef PF_W
+#  define PF_W         2
+#  endif
 #else
 #  include <elf.h>
 #endif
@@ -565,7 +442,6 @@ static struct link_map *
 GC_FirstDLOpenedLinkMap()
 {
     ElfW(Dyn) *dp;
-    struct r_debug *r;
     static struct link_map *cachedResult = 0;
 
     if( _DYNAMIC == 0) {
@@ -656,16 +532,16 @@ void GC_register_dynamic_libraries()
     static prmap_t * addr_map = 0;
     static int current_sz = 0;	/* Number of records currently in addr_map */
     static int needed_sz;	/* Required size of addr_map		*/
-    register int i;
-    register long flags;
-    register ptr_t start;
-    register ptr_t limit;
+    int i;
+    long flags;
+    ptr_t start;
+    ptr_t limit;
     ptr_t heap_start = (ptr_t)HEAP_START;
     ptr_t heap_end = heap_start;
 
-#   ifdef SUNOS5DL
+#   ifdef SOLARISDL
 #     define MA_PHYS 0
-#   endif /* SUNOS5DL */
+#   endif /* SOLARISDL */
 
     if (fd < 0) {
       sprintf(buf, "/proc/%d", getpid());
@@ -677,7 +553,7 @@ void GC_register_dynamic_libraries()
       }
     }
     if (ioctl(fd, PIOCNMAP, &needed_sz) < 0) {
-	GC_err_printf2("fd = %d, errno = %d\n", fd, errno);
+	GC_err_printf("fd = %d, errno = %d\n", fd, errno);
     	ABORT("/proc PIOCNMAP ioctl failed");
     }
     if (needed_sz >= current_sz) {
@@ -687,7 +563,7 @@ void GC_register_dynamic_libraries()
 						(current_sz * sizeof(prmap_t)));
     }
     if (ioctl(fd, PIOCMAP, addr_map) < 0) {
-        GC_err_printf4("fd = %d, errno = %d, needed_sz = %d, addr_map = 0x%X\n",
+        GC_err_printf("fd = %d, errno = %d, needed_sz = %d, addr_map = 0x%X\n",
                         fd, errno, needed_sz, addr_map);
     	ABORT("/proc PIOCMAP ioctl failed");
     };
@@ -698,7 +574,8 @@ void GC_register_dynamic_libraries()
     }
     for (i = 0; i < needed_sz; i++) {
         flags = addr_map[i].pr_mflags;
-        if ((flags & (MA_BREAK | MA_STACK | MA_PHYS)) != 0) goto irrelevant;
+	if ((flags & (MA_BREAK | MA_STACK | MA_PHYS
+		      | MA_FETCHOP | MA_NOTCACHED)) != 0) goto irrelevant;
         if ((flags & (MA_READ | MA_WRITE)) != (MA_READ | MA_WRITE))
             goto irrelevant;
           /* The latter test is empirically useless in very old Irix	*/
@@ -759,7 +636,7 @@ void GC_register_dynamic_libraries()
 
 # endif /* USE_PROC || IRIX5 */
 
-# if defined(MSWIN32) || defined(MSWINCE)
+# if defined(MSWIN32) || defined(MSWINCE) || defined(CYGWIN32)
 
 # define WIN32_LEAN_AND_MEAN
 # define NOSERVICE
@@ -820,10 +697,26 @@ void GC_register_dynamic_libraries()
   
 # define HAVE_REGISTER_MAIN_STATIC_DATA
 
+# ifdef DEBUG_VIRTUALQUERY
+  void GC_dump_meminfo(MEMORY_BASIC_INFORMATION *buf)
+  {
+    GC_printf("BaseAddress = %lx, AllocationBase = %lx, RegionSize = %lx(%lu)\n",
+	       buf -> BaseAddress, buf -> AllocationBase, buf -> RegionSize,
+	       buf -> RegionSize);
+    GC_printf("\tAllocationProtect = %lx, State = %lx, Protect = %lx, "
+	       "Type = %lx\n",
+	       buf -> AllocationProtect, buf -> State, buf -> Protect,
+	       buf -> Type);
+  }
+# endif /* DEBUG_VIRTUALQUERY */
+
+  extern GC_bool GC_wnt;  /* Is Windows NT derivative.		*/
+  			  /* Defined and set in os_dep.c.	*/
+
   void GC_register_dynamic_libraries()
   {
     MEMORY_BASIC_INFORMATION buf;
-    DWORD result;
+    size_t result;
     DWORD protect;
     LPVOID p;
     char * base;
@@ -834,8 +727,8 @@ void GC_register_dynamic_libraries()
 #   endif
     base = limit = p = GC_sysinfo.lpMinimumApplicationAddress;
 #   if defined(MSWINCE) && !defined(_WIN32_WCE_EMULATION)
-    /* Only the first 32 MB of address space belongs to the current process */
-    while (p < (LPVOID)0x02000000) {
+      /* Only the first 32 MB of address space belongs to the current process */
+      while (p < (LPVOID)0x02000000) {
         result = VirtualQuery(p, &buf, sizeof(buf));
 	if (result == 0) {
 	    /* Page is free; advance to the next possible allocation base */
@@ -844,7 +737,7 @@ void GC_register_dynamic_libraries()
 		 & ~(GC_sysinfo.dwAllocationGranularity-1));
 	} else
 #   else
-    while (p < GC_sysinfo.lpMaximumApplicationAddress) {
+      while (p < GC_sysinfo.lpMaximumApplicationAddress) {
         result = VirtualQuery(p, &buf, sizeof(buf));
 #   endif
 	{
@@ -856,7 +749,16 @@ void GC_register_dynamic_libraries()
 	    if (buf.State == MEM_COMMIT
 		&& (protect == PAGE_EXECUTE_READWRITE
 		    || protect == PAGE_READWRITE)
-		&& !GC_is_heap_base(buf.AllocationBase)) {
+		&& !GC_is_heap_base(buf.AllocationBase)
+ 		/* There is some evidence that we cannot always
+ 		 * ignore MEM_PRIVATE sections under Windows ME
+ 		 * and predecessors.  Hence we now also check for
+ 		 * that case.	*/
+ 		&& (buf.Type == MEM_IMAGE ||
+ 		    !GC_wnt && buf.Type == MEM_PRIVATE)) {
+#	        ifdef DEBUG_VIRTUALQUERY
+	          GC_dump_meminfo(&buf);
+#	        endif
 		if ((char *)p != limit) {
 		    GC_cond_add_roots(base, limit);
 		    base = p;
@@ -870,7 +772,7 @@ void GC_register_dynamic_libraries()
     GC_cond_add_roots(base, limit);
   }
 
-#endif /* MSWIN32 || MSWINCE */
+#endif /* MSWIN32 || MSWINCE || CYGWIN32 */
   
 #if defined(ALPHA) && defined(OSF1)
 
@@ -909,15 +811,15 @@ void GC_register_dynamic_libraries()
       /* Check status AFTER checking moduleid because */
       /* of a bug in the non-shared ldr_next_module stub */
         if (status != 0 ) {
-            GC_printf1("dynamic_load: status = %ld\n", (long)status);
+            GC_printf("dynamic_load: status = %d\n", status);
             {
                 extern char *sys_errlist[];
                 extern int sys_nerr;
                 extern int errno;
                 if (errno <= sys_nerr) {
-                    GC_printf1("dynamic_load: %s\n", (long)sys_errlist[errno]);
+                    GC_printf("dynamic_load: %s\n", sys_errlist[errno]);
                } else {
-                    GC_printf1("dynamic_load: %d\n", (long)errno);
+                    GC_printf("dynamic_load: %d\n", errno);
                 }
         }
             ABORT("ldr_next_module failed");
@@ -933,7 +835,7 @@ void GC_register_dynamic_libraries()
           if (moduleinfo.lmi_flags & LDR_MAIN)
               continue;    /* skip the main module */
 
-#     ifdef VERBOSE
+#     ifdef DL_VERBOSE
           GC_printf("---Module---\n");
           GC_printf("Module ID            = %16ld\n", moduleinfo.lmi_modid);
           GC_printf("Count of regions     = %16d\n", moduleinfo.lmi_nregion);
@@ -954,7 +856,7 @@ void GC_register_dynamic_libraries()
             if (! (regioninfo.lri_prot & LDR_W))
                 continue;
 
-#         ifdef VERBOSE
+#         ifdef DL_VERBOSE
               GC_printf("--- Region ---\n");
               GC_printf("Region number    = %16ld\n",
               	        regioninfo.lri_region_no);
@@ -981,7 +883,6 @@ void GC_register_dynamic_libraries()
 #include <errno.h>
 #include <dl.h>
 
-extern int errno;
 extern char *sys_errlist[];
 extern int sys_nerr;
 
@@ -1008,26 +909,26 @@ void GC_register_dynamic_libraries()
               break; /* Moved past end of shared library list --> finished */
           } else {
               if (errno <= sys_nerr) {
-                    GC_printf1("dynamic_load: %s\n", (long) sys_errlist[errno]);
+                    GC_printf("dynamic_load: %s\n", sys_errlist[errno]);
               } else {
-                    GC_printf1("dynamic_load: %d\n", (long) errno);
+                    GC_printf("dynamic_load: %d\n", errno);
 	      }
               ABORT("shl_get failed");
           }
 #	 endif
         }
 
-#     ifdef VERBOSE
-          GC_printf0("---Shared library---\n");
-          GC_printf1("\tfilename        = \"%s\"\n", shl_desc->filename);
-          GC_printf1("\tindex           = %d\n", index);
-          GC_printf1("\thandle          = %08x\n",
+#     ifdef DL_VERBOSE
+          GC_printf("---Shared library---\n");
+          GC_printf("\tfilename        = \"%s\"\n", shl_desc->filename);
+          GC_printf("\tindex           = %d\n", index);
+          GC_printf("\thandle          = %08x\n",
 					(unsigned long) shl_desc->handle);
-          GC_printf1("\ttext seg. start = %08x\n", shl_desc->tstart);
-          GC_printf1("\ttext seg. end   = %08x\n", shl_desc->tend);
-          GC_printf1("\tdata seg. start = %08x\n", shl_desc->dstart);
-          GC_printf1("\tdata seg. end   = %08x\n", shl_desc->dend);
-          GC_printf1("\tref. count      = %lu\n", shl_desc->ref_count);
+          GC_printf("\ttext seg. start = %08x\n", shl_desc->tstart);
+          GC_printf("\ttext seg. end   = %08x\n", shl_desc->tend);
+          GC_printf("\tdata seg. start = %08x\n", shl_desc->dstart);
+          GC_printf("\tdata seg. end   = %08x\n", shl_desc->dend);
+          GC_printf("\tref. count      = %lu\n", shl_desc->ref_count);
 #     endif
 
       /* register shared library's data segment as a garbage collection root */
@@ -1039,7 +940,7 @@ void GC_register_dynamic_libraries()
 }
 #endif /* HPUX */
 
-#ifdef RS6000
+#ifdef AIX
 #pragma alloca
 #include <sys/ldr.h>
 #include <sys/errno.h>
@@ -1064,71 +965,162 @@ void GC_register_dynamic_libraries()
 		len = ldi->ldinfo_next;
 		GC_add_roots_inner(
 				ldi->ldinfo_dataorg,
-				(unsigned long)ldi->ldinfo_dataorg
+				(ptr_t)(unsigned long)ldi->ldinfo_dataorg
 			        + ldi->ldinfo_datasize,
 				TRUE);
 		ldi = len ? (struct ld_info *)((char *)ldi + len) : 0;
 	}
 }
-#endif /* RS6000 */
+#endif /* AIX */
 
-#ifdef MACOSX
+#ifdef DARWIN
 
-#include <mach-o/dyld.h>
+/* __private_extern__ hack required for pre-3.4 gcc versions.	*/
+#ifndef __private_extern__
+# define __private_extern__ extern
+# include <mach-o/dyld.h>
+# undef __private_extern__
+#else
+# include <mach-o/dyld.h>
+#endif
 #include <mach-o/getsect.h>
 
-/*#define MACOSX_DEBUG */
+/*#define DARWIN_DEBUG*/
 
-void GC_register_dynamic_libraries() 
-{
-    unsigned long image_count;
-    const struct mach_header *mach_header;
-    const struct section *sec;
-    unsigned long slide;
-    unsigned long filetype;
-    int i,j;
-    unsigned long start;
-    unsigned long end;
-    
-    static struct { 
+const static struct { 
         const char *seg;
         const char *sect;
-    } sections[] = {
+} GC_dyld_sections[] = {
         { SEG_DATA, SECT_DATA },
         { SEG_DATA, SECT_BSS },
         { SEG_DATA, SECT_COMMON }
-    };
+};
     
-    image_count = _dyld_image_count();
-    for(i=0;i<image_count;i++)
-    {
-        mach_header = _dyld_get_image_header(i);
-        slide = _dyld_get_image_vmaddr_slide(i);
-        filetype = mach_header->filetype;
+#ifdef DARWIN_DEBUG
+static const char *GC_dyld_name_for_hdr(const struct GC_MACH_HEADER *hdr) {
+    unsigned long i,c;
+    c = _dyld_image_count();
+    for(i=0;i<c;i++) if(_dyld_get_image_header(i) == hdr)
+        return _dyld_get_image_name(i);
+    return NULL;
+}
+#endif
         
-        for(j=0;j<sizeof(sections)/sizeof(sections[0]);j++) {
-            sec = getsectbynamefromheader(mach_header,sections[j].seg,sections[j].sect);
-            if(sec == NULL || sec->size == 0) continue;
-            start = slide + sec->addr;
-            end = start + sec->size;
-#			ifdef MACOSX_DEBUG
-                GC_printf4("Adding section at %p-%p (%lu bytes) from image %s\n",
-                    start,end,sec->size,_dyld_get_image_name(i));
-#			endif
-
-            GC_add_roots_inner((char*)start,(char*)end,
-                filetype == MH_EXECUTE ? FALSE : TRUE);
-        }
+/* This should never be called by a thread holding the lock */
+static void GC_dyld_image_add(const struct GC_MACH_HEADER *hdr, intptr_t slide)
+{
+    unsigned long start,end,i;
+    const struct GC_MACH_SECTION *sec;
+    if (GC_no_dls) return;
+    for(i=0;i<sizeof(GC_dyld_sections)/sizeof(GC_dyld_sections[0]);i++) {
+#   if defined (__LP64__)
+      sec = getsectbynamefromheader_64(
+#   else
+      sec = getsectbynamefromheader(
+#   endif
+				    hdr, GC_dyld_sections[i].seg,
+				    GC_dyld_sections[i].sect);
+      if(sec == NULL || sec->size == 0) continue;
+      start = slide + sec->addr;
+      end = start + sec->size;
+#   ifdef DARWIN_DEBUG
+      GC_printf("Adding section at %p-%p (%lu bytes) from image %s\n",
+		start,end,sec->size,GC_dyld_name_for_hdr(hdr));
+#   endif
+      GC_add_roots((char*)start,(char*)end);
     }
+#   ifdef DARWIN_DEBUG
+       GC_print_static_roots();
+#   endif
+}
+
+/* This should never be called by a thread holding the lock */
+static void GC_dyld_image_remove(const struct GC_MACH_HEADER *hdr,
+				 intptr_t slide)
+{
+    unsigned long start,end,i;
+    const struct GC_MACH_SECTION *sec;
+    for(i=0;i<sizeof(GC_dyld_sections)/sizeof(GC_dyld_sections[0]);i++) {
+#   if defined (__LP64__)
+      sec = getsectbynamefromheader_64(
+#   else
+      sec = getsectbynamefromheader(
+#   endif
+				    hdr, GC_dyld_sections[i].seg,
+				    GC_dyld_sections[i].sect);
+      if(sec == NULL || sec->size == 0) continue;
+      start = slide + sec->addr;
+      end = start + sec->size;
+#   ifdef DARWIN_DEBUG
+      GC_printf("Removing section at %p-%p (%lu bytes) from image %s\n",
+		start,end,sec->size,GC_dyld_name_for_hdr(hdr));
+#   endif
+      GC_remove_roots((char*)start,(char*)end);
+    }
+#   ifdef DARWIN_DEBUG
+	GC_print_static_roots();
+#   endif
+}
+
+void GC_register_dynamic_libraries() {
+    /* Currently does nothing. The callbacks are setup by GC_init_dyld() 
+    The dyld library takes it from there. */
+}
+
+/* The _dyld_* functions have an internal lock so no _dyld functions
+   can be called while the world is stopped without the risk of a deadlock.
+   Because of this we MUST setup callbacks BEFORE we ever stop the world.
+   This should be called BEFORE any thread in created and WITHOUT the
+   allocation lock held. */
+   
+void GC_init_dyld() {
+  static GC_bool initialized = FALSE;
+  char *bind_fully_env = NULL;
+  
+  if(initialized) return;
+  
+#   ifdef DARWIN_DEBUG
+      GC_printf("Registering dyld callbacks...\n");
+#   endif
+  
+  /* Apple's Documentation:
+     When you call _dyld_register_func_for_add_image, the dynamic linker runtime
+     calls the specified callback (func) once for each of the images that is
+     currently loaded into the program. When a new image is added to the program,
+     your callback is called again with the mach_header for the new image, and the 	
+     virtual memory slide amount of the new image. 
+     
+     This WILL properly register already linked libraries and libraries 
+     linked in the future
+  */
+  
+    _dyld_register_func_for_add_image(GC_dyld_image_add);
+    _dyld_register_func_for_remove_image(GC_dyld_image_remove);
+
+    /* Set this early to avoid reentrancy issues. */
+    initialized = TRUE;
+
+    bind_fully_env = getenv("DYLD_BIND_AT_LAUNCH");
+    
+    if (bind_fully_env == NULL) {
+#   ifdef DARWIN_DEBUG
+      GC_printf("Forcing full bind of GC code...\n");
+#   endif
+      
+      if(!_dyld_bind_fully_image_containing_address((unsigned long*)GC_malloc))
+        GC_abort("_dyld_bind_fully_image_containing_address failed");
+    }
+
 }
 
 #define HAVE_REGISTER_MAIN_STATIC_DATA
 GC_bool GC_register_main_static_data()
 {
+  /* Already done through dyld callbacks */
   return FALSE;
 }
 
-#endif /* MACOSX */
+#endif /* DARWIN */
 
 #else /* !DYNAMIC_LOADING */
 
@@ -1186,5 +1178,13 @@ GC_bool GC_register_main_static_data()
 {
   return TRUE;
 }
+
+/* Register a routine to filter dynamic library registration.  */
+void
+GC_register_has_static_roots_callback
+  (int (*callback)(const char *, void *, size_t)) {
+  GC_has_static_roots = callback;
+}
+
 #endif /* HAVE_REGISTER_MAIN_STATIC_DATA */
 
