@@ -18,7 +18,7 @@ using namespace Synopsis;
 
 int usage(const char *command)
 {
-  std::cerr << "Usage: " << command << " [-c] [-g <filename>] [-d] [-r] [-e] <input>" << std::endl;
+  std::cerr << "Usage: " << command << " [-c] [-g <filename>] [-d] [-r]  [-s] [-e] <input>" << std::endl;
   return -1;
 }
 
@@ -27,6 +27,7 @@ int main(int argc, char **argv)
   bool encoding = false;
   bool typeinfo = false;
   bool debug = false;
+  SymbolFactory::Language language = SymbolFactory::NONE;
   int tokenset = Lexer::CXX | Lexer::GCC;
   int ruleset = Parser::CXX | Parser::GCC;
   std::string dotfile;
@@ -47,8 +48,14 @@ int main(int argc, char **argv)
     else if (argv[i] == std::string("-c"))
     {
       tokenset = Lexer::C | Lexer::GCC;
+      if (language != SymbolFactory::NONE)
+        language = SymbolFactory::CXX;
       ruleset = Parser::GCC;
     }
+    else if (argv[i] == std::string("-s"))
+    {
+      language = tokenset & Lexer::CXX ? SymbolFactory::CXX : SymbolFactory::C99;
+    }
     else if (argv[i] == std::string("-e")) encoding = true;
     else if (argv[i] == std::string("-r")) typeinfo = true;
     else if (argv[i] == std::string("-d")) debug = true;
@@ -64,7 +71,7 @@ int main(int argc, char **argv)
     std::ifstream ifs(input);
     Buffer buffer(ifs.rdbuf(), input);
     Lexer lexer(&buffer, tokenset);
-    SymbolFactory symbols;
+    SymbolFactory symbols(language);
     Parser parser(lexer, symbols, ruleset);
     PTree::Node *node = parser.parse();
     const Parser::ErrorList &errors = parser.errors();
